/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 */


#include "prtypes.h"
#include "nsIContent.h"
#include "nsIAtom.h"
#include "nsIPresContext.h"
#include "nsIStyleContext.h"
#include "nsINameSpaceManager.h"

#include "nsString.h"
#include "nsCSSRendering.h"
#include "nsFrame.h"

#include "nsSVGFrames.h"
#include "nsSVGAtoms.h"


class nsSVGPathFrame : public nsFrame
{
public:
  nsSVGPathFrame() {};
  ~nsSVGPathFrame() {};
  
    // nsIFrame overrides
  NS_IMETHOD GetFrameName(nsString& aResult) const {
    return MakeFrameName("SVGPathFrame", aResult);
  }
  NS_IMETHOD Paint(nsIPresContext* aPresContext,
                   nsIRenderingContext& aRenderingContext,
                   const nsRect& aDirtyRect,
                   nsFramePaintLayer aWhichLayer);
  
protected:

};

//
// NS_NewSVGPathFrame
//
// Wrapper for creating a new SVG Path
//
nsresult
NS_NewSVGPathFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
{
  NS_PRECONDITION(aNewFrame, "null OUT ptr");
  if (nsnull == aNewFrame) {
    return NS_ERROR_NULL_POINTER;
  }
  nsSVGPathFrame* it = new (aPresShell) nsSVGPathFrame;
  if ( !it )
    return NS_ERROR_OUT_OF_MEMORY;
  *aNewFrame = it;
  return NS_OK;
}


NS_METHOD 
nsSVGPathFrame::Paint(nsIPresContext* aPresContext,
                              nsIRenderingContext& aRenderingContext,
                              const nsRect& aDirtyRect,
                              nsFramePaintLayer aWhichLayer)
{
  float p2t;
  aPresContext->GetPixelsToTwips(&p2t);

  nsString d, c, x, y;
  PRInt32 err;
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::d, d);

  /* Turn the string into "Cx,yCx,y;x,y;" */
  d.CompressWhitespace(PR_TRUE, PR_TRUE);
  d.Append(' ');
  PRInt32 io = 0, ix = 0, iy = 0;
  PRInt32 p = 0, points = 0;
  while (0 <= (iy = d.Find(" ", PR_FALSE, iy))) {
    if (d.IsDigit(d.CharAt(iy)))
	d.SetCharAt(';',iy++);
    points++;
  }
  d.StripWhitespace();

  nsPoint * verts = new nsPoint[points];
  while (0 <= (ix = d.Find(",", PR_FALSE, io))) {
    iy = d.Find(";", PR_FALSE, ix);
    d.Mid(x, io, ix-io);
    io = ix + 1;
    d.Mid(y, io, iy-io);  
    io = iy + 1;    
    verts[p].x = NSIntPixelsToTwips(x.ToInteger(&err), p2t);
    verts[p].y = NSIntPixelsToTwips(y.ToInteger(&err), p2t);
    p++;
  }

  const nsStyleColor * color = (nsStyleColor *)
	mStyleContext->GetStyleData(eStyleStruct_Color);
  
  aRenderingContext.SetColor(color->mColor);
  aRenderingContext.DrawPolyline(verts, points);
  aRenderingContext.SetColor(color->mBackgroundColor);
  aRenderingContext.FillPolygon(verts, points);
  return NS_OK;
}

