/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 */


#include "prtypes.h"
#include "xp_core.h"
#include "nsIContent.h"
#include "nsIAtom.h"
#include "nsIPresContext.h"
#include "nsIStyleContext.h"
#include "nsINameSpaceManager.h"

#include "nsString.h"
#include "nsCSSRendering.h"
#include "nsFrame.h"

#include "nsSVGFrames.h"
#include "nsSVGAtoms.h"


class nsSVGPolygonFrame : public nsFrame
{
public:
  nsSVGPolygonFrame() {};
  ~nsSVGPolygonFrame() {};
  
    // nsIFrame overrides
  NS_IMETHOD GetFrameName(nsString& aResult) const {
    return MakeFrameName("SVGPolygonFrame", aResult);
  }
  NS_IMETHOD Paint(nsIPresContext* aPresContext,
                   nsIRenderingContext& aRenderingContext,
                   const nsRect& aDirtyRect,
                   nsFramePaintLayer aWhichLayer);
  
protected:

};

//
// NS_NewSVGPolygonFrame
//
// Wrapper for creating a new SVG Polygon
//
nsresult
NS_NewSVGPolygonFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
{
  NS_PRECONDITION(aNewFrame, "null OUT ptr");
  if (nsnull == aNewFrame) {
    return NS_ERROR_NULL_POINTER;
  }
  nsSVGPolygonFrame* it = new (aPresShell) nsSVGPolygonFrame;
  if ( !it )
    return NS_ERROR_OUT_OF_MEMORY;
  *aNewFrame = it;
  return NS_OK;
}


NS_METHOD 
nsSVGPolygonFrame::Paint(nsIPresContext* aPresContext,
                              nsIRenderingContext& aRenderingContext,
                              const nsRect& aDirtyRect,
                              nsFramePaintLayer aWhichLayer)
{
  float p2t;
  aPresContext->GetPixelsToTwips(&p2t);

  nsString v;
  PRInt32 err;
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::cx, v);
  nscoord cx = NSIntPixelsToTwips(v.ToInteger(&err), p2t);
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::cy, v);
  nscoord cy = NSIntPixelsToTwips(v.ToInteger(&err), p2t);
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::sides, v);
  PRUint32 sides = v.ToInteger(&err);
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::angle, v);
  PRFloat64 angle = v.ToInteger(&err)*(M_PI/180.0);
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::r, v);
  nscoord r = NSIntPixelsToTwips(v.ToInteger(&err), p2t);
  const nsStyleColor * color = (nsStyleColor *)
	mStyleContext->GetStyleData(eStyleStruct_Color);

  nsPoint * verts = new nsPoint[sides+1];
  for (int i = 0; i <= sides; i++) {
    PRFloat64 radial = (2*M_PI*i)/sides + angle;
    verts[i].x = cx + r*sin(radial);
    verts[i].y = cy + r*cos(radial);
  }
  
  aRenderingContext.SetColor(color->mColor);
  aRenderingContext.FillPolygon(verts, sides+1);

  delete [] verts;

  return NS_OK;
}
