/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 */


#include "prtypes.h"
#include "nsIContent.h"
#include "nsIAtom.h"
#include "nsIPresContext.h"
#include "nsIStyleContext.h"
#include "nsINameSpaceManager.h"

#include "nsString.h"
#include "nsCSSRendering.h"
#include "nsFrame.h"

#include "nsSVGFrames.h"
#include "nsSVGAtoms.h"


class nsSVGPolylineFrame : public nsFrame
{
public:
  nsSVGPolylineFrame() {};
  ~nsSVGPolylineFrame() {};
  
    // nsIFrame overrides
  NS_IMETHOD GetFrameName(nsString& aResult) const {
    return MakeFrameName("SVGPolylineFrame", aResult);
  }
  NS_IMETHOD Paint(nsIPresContext* aPresContext,
                   nsIRenderingContext& aRenderingContext,
                   const nsRect& aDirtyRect,
                   nsFramePaintLayer aWhichLayer);
  
protected:

};

//
// NS_NewSVGPolylineFrame
//
// Wrapper for creating a new SVG Polyline
//
nsresult
NS_NewSVGPolylineFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
{
  NS_PRECONDITION(aNewFrame, "null OUT ptr");
  if (nsnull == aNewFrame) {
    return NS_ERROR_NULL_POINTER;
  }
  nsSVGPolylineFrame* it = new (aPresShell) nsSVGPolylineFrame;
  if ( !it )
    return NS_ERROR_OUT_OF_MEMORY;
  *aNewFrame = it;
  return NS_OK;
}


NS_METHOD 
nsSVGPolylineFrame::Paint(nsIPresContext* aPresContext,
                              nsIRenderingContext& aRenderingContext,
                              const nsRect& aDirtyRect,
                              nsFramePaintLayer aWhichLayer)
{
  float p2t;
  aPresContext->GetPixelsToTwips(&p2t);

  nsString v, x, y;
  PRInt32 err;
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::verts, v);

  /* Turn the string into "x,y;x,y;x,y;", counting as we go. */
  v.CompressWhitespace(PR_TRUE, PR_TRUE);
  v.Append(' ');
  PRInt32 io = 0, ix = 0, iy = 0;
  PRInt32 p = 0, points = 0;
  while (0 <= (iy = v.Find(" ", PR_FALSE, iy))) {
    v.SetCharAt(';',iy++);
    points++;
  }

  nsPoint * verts = new nsPoint[points];
  while (0 <= (ix = v.Find(",", PR_FALSE, io))) {
    iy = v.Find(";", PR_FALSE, ix);
    v.Mid(x, io, ix-io);
    io = ix + 1;
    v.Mid(y, io, iy-io);  
    io = iy + 1;    
    verts[p].x = NSIntPixelsToTwips(x.ToInteger(&err), p2t);
    verts[p].y = NSIntPixelsToTwips(y.ToInteger(&err), p2t);
    p++;
  }

  nsStyleColor color;
	mStyleContext->GetStyle(eStyleStruct_Color, color);
  
  aRenderingContext.SetColor(color.mColor);
  aRenderingContext.DrawPolyline(verts, points);
  aRenderingContext.SetColor(color.mBackgroundColor);
	printf("Color :%i %i %i\n", 
		NS_GET_R(color.mBackgroundColor),
		NS_GET_G(color.mBackgroundColor),
		NS_GET_B(color.mBackgroundColor));
  aRenderingContext.FillPolygon(verts, points);
  return NS_OK;
}
