/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 */


#include "prtypes.h"
#include "nsIContent.h"
#include "nsIAtom.h"
#include "nsIPresContext.h"
#include "nsIStyleContext.h"
#include "nsINameSpaceManager.h"

#include "nsString.h"
#include "nsCSSRendering.h"
#include "nsFrame.h"

#include "nsSVGFrames.h"
#include "nsSVGAtoms.h"

#include "nsSVGPathSeg.h"

class nsSVGRectFrame : public nsFrame
{
public:
  nsSVGRectFrame() {};
  ~nsSVGRectFrame() {};
  
    // nsIFrame overrides
  NS_IMETHOD GetFrameName(nsString& aResult) const {
    return MakeFrameName("SVGRectFrame", aResult);
  }
  NS_IMETHOD Paint(nsIPresContext* aPresContext,
                   nsIRenderingContext& aRenderingContext,
                   const nsRect& aDirtyRect,
                   nsFramePaintLayer aWhichLayer);
  
protected:

};

//
// NS_NewSVGRectFrame
//
// Wrapper for creating a new SVG Rect
//
nsresult
NS_NewSVGRectFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
{
  NS_PRECONDITION(aNewFrame, "null OUT ptr");
  if (nsnull == aNewFrame) {
    return NS_ERROR_NULL_POINTER;
  }
  nsSVGRectFrame* it = new (aPresShell) nsSVGRectFrame;
  if ( !it )
    return NS_ERROR_OUT_OF_MEMORY;
  *aNewFrame = it;
  return NS_OK;
}


NS_METHOD 
nsSVGRectFrame::Paint(nsIPresContext* aPresContext,
											nsIRenderingContext& aRenderingContext,
											const nsRect& aDirtyRect,
											nsFramePaintLayer aWhichLayer)
{
  float p2t;
  aPresContext->GetPixelsToTwips(&p2t);

  nsString v;
  PRInt32 err;
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::x, v);
  nscoord x = NSIntPixelsToTwips(v.ToInteger(&err), p2t);
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::y, v);
  nscoord y = NSIntPixelsToTwips(v.ToInteger(&err), p2t);
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::width, v);
  nscoord width = NSIntPixelsToTwips(v.ToInteger(&err), p2t);
  mContent->GetAttribute(kNameSpaceID_None, nsSVGAtoms::height, v);
  nscoord height = NSIntPixelsToTwips(v.ToInteger(&err), p2t);
  const nsStyleColor * color = (nsStyleColor *)
	mStyleContext->GetStyleData(eStyleStruct_Color);

  aRenderingContext.SetColor(color->mColor);
  aRenderingContext.FillRect(x, y, width, height);
  return NS_OK;
}
