/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation.  Portions created by Netscape are Copyright (C) 1998
 * Netscape Communications Corporation.  All Rights Reserved.
 */

#include "nsSVGElement.h"
#include "nsIDocument.h"
#include "nsIAtom.h"
#include "nsIEventListenerManager.h"
#include "nsIDOMScriptObjectFactory.h"
#include "nsHTMLParts.h"
#include "nsIEventStateManager.h"
#include "nsDOMEvent.h"
#include "nsINameSpace.h"
#include "nsINameSpaceManager.h"
#include "nsIURL.h"
#include "nsICSSParser.h"
#include "nsIServiceManager.h"
#include "nsLayoutCID.h"
#include "nsStyleConsts.h"
#include "nsISupportsArray.h"
#include "nsIStyleRule.h"

#include "nsSVGAtoms.h"
#include "nsSVGFrames.h"

//static NS_DEFINE_IID(kIDOMElementIID, NS_IDOMELEMENT_IID);
static NS_DEFINE_IID(kIXMLContentIID, NS_IXMLCONTENT_IID);
static NS_DEFINE_CID(kCSSParserCID, NS_CSSPARSER_CID);
static NS_DEFINE_CID(kICSSParserIID, NS_ICSS_PARSER_IID);

nsresult
NS_NewSVGElement(nsIXMLContent** aInstancePtrResult, nsIAtom* aTag)
{
  NS_PRECONDITION(nsnull != aInstancePtrResult, "null ptr");
  if (nsnull == aInstancePtrResult) {
    return NS_ERROR_NULL_POINTER;
  }
  nsIXMLContent* it = new nsSVGElement(aTag);
  if (nsnull == it) {
    return NS_ERROR_OUT_OF_MEMORY;
  }
  return it->QueryInterface(kIXMLContentIID, (void**) aInstancePtrResult);
}

nsSVGElement::nsSVGElement(nsIAtom *aTag)
{
  printf("nsSVGElement::nsSVGElement()\n");
  NS_INIT_REFCNT();
  mInner.Init((nsIContent *)(nsIXMLContent *)this, aTag);
  mIsLink = PR_FALSE;
}
 
nsSVGElement::~nsSVGElement()
{
  printf("nsSVGElement::~nsSVGElement()\n");

}

NS_IMETHODIMP 
nsSVGElement::QueryInterface(REFNSIID aIID,
			     void** aInstancePtr)
{
  NS_IMPL_CONTENT_QUERY_INTERFACE(aIID, aInstancePtr, this, nsIXMLContent)
  if (aIID.Equals(nsIStyledContent::GetIID()) ||
      aIID.Equals(kIContentIID) ||
      aIID.Equals(kISupportsIID)) {
    nsIStyledContent* tmp = this;
    *aInstancePtr = (void*) tmp;
    NS_ADDREF_THIS();
    return NS_OK;
  }
  else if (aIID.Equals(kIXMLContentIID)) {
    nsIXMLContent* tmp = this;
    *aInstancePtr = (void*) tmp;
    NS_ADDREF_THIS();
    return NS_OK;
  }
  
  return NS_NOINTERFACE;
}


NS_IMPL_ADDREF(nsSVGElement)
NS_IMPL_RELEASE(nsSVGElement)


NS_IMETHODIMP 
nsSVGElement::SetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName, 
			                     const nsString& aValue,
			                     PRBool aNotify)
{
/*  // XXX It sucks that we have to do a strcmp for
  // every attribute set. It might be a bit more expensive
  // to create an atom.
  if ((kNameSpaceID_XML == aNameSpaceID) && 
      (aName == kLinkAtom) && (aValue.Equals("simple"))) {
    mIsLink = PR_TRUE;
  } */

  return mInner.SetAttribute(aNameSpaceID, aName, aValue, aNotify);
}

NS_IMETHODIMP 
nsSVGElement::HandleDOMEvent(nsIPresContext* aPresContext,
			     nsEvent* aEvent,
			     nsIDOMEvent** aDOMEvent,
			     PRUint32 aFlags,
			     nsEventStatus* aEventStatus)
{
  // Try script event handlers first
  nsresult ret = mInner.HandleDOMEvent(aPresContext, aEvent, aDOMEvent,
                                       aFlags, aEventStatus);
/*
  if (mIsLink && (NS_OK == ret) && (nsEventStatus_eIgnore == aEventStatus)) {
    switch (aEvent->message) {
    case NS_MOUSE_LEFT_BUTTON_DOWN:
      {
        nsIEventStateManager *stateManager;
        if (NS_OK == aPresContext.GetEventStateManager(&stateManager)) {
          stateManager->SetContentState(this, NS_EVENT_STATE_ACTIVE | NS_EVENT_STATE_FOCUS);
          NS_RELEASE(stateManager);
        }
        aEventStatus = nsEventStatus_eConsumeDoDefault; 
      }
      break;

    case NS_MOUSE_LEFT_CLICK:
      {
        if (nsEventStatus_eConsumeNoDefault != aEventStatus) {
          nsAutoString show, href, target;
          nsIURI* baseURL = nsnull;
	        nsLinkVerb verb = eLinkVerb_Replace;
	        target.Truncate();
          GetAttribute(kNameSpaceID_None, kHrefAtom, href);
          GetAttribute(kNameSpaceID_None, kShowAtom, show);
	        // XXX Should probably do this using atoms 
	        if (show.Equals("new")) {
	          verb = eLinkVerb_New;
	        }
	        else if (show.Equals("embed")) {
	          verb = eLinkVerb_Embed;
	        }
          
          if (nsnull != mInner.mDocument) {
            baseURL = mInner.mDocument->GetDocumentURL();
          }
          mInner.TriggerLink(aPresContext, verb, baseURL, href, target, PR_TRUE);
          NS_IF_RELEASE(baseURL);
          aEventStatus = nsEventStatus_eConsumeDoDefault; 
        }
      }
      break;

    case NS_MOUSE_RIGHT_BUTTON_DOWN:
      // XXX Bring up a contextual menu provided by the application
      break;

    case NS_MOUSE_ENTER:
      {
        nsAutoString href, target;
        nsIURI* baseURL = nsnull;
        GetAttribute(kNameSpaceID_None, kHrefAtom, href);
        if (nsnull != mInner.mDocument) {
          baseURL = mInner.mDocument->GetDocumentURL();
        }

        mInner.TriggerLink(aPresContext, eLinkVerb_Replace, baseURL, href, target, PR_FALSE);
        
        NS_IF_RELEASE(baseURL);
        aEventStatus = nsEventStatus_eConsumeDoDefault; 
      }
      break;

      // XXX this doesn't seem to do anything yet
    case NS_MOUSE_EXIT:
      {
        nsAutoString empty;
        mInner.TriggerLink(aPresContext, eLinkVerb_Replace, nsnull, empty, empty, PR_FALSE);
        aEventStatus = nsEventStatus_eConsumeDoDefault; 
      }
      break;

    default:
      break;
    }
  } */
  return ret;
}

NS_IMETHODIMP
nsSVGElement::CreateFrame(nsIPresShell* aPresShell, nsIFrame** aFrame)
{
  nsresult rv = NS_OK;

  printf("nsSVGElement::CreateFrame()\n");
  if (mInner.mTag == nsSVGAtoms::rect)
	rv = NS_NewSVGRectFrame(aPresShell, aFrame);
  else if (mInner.mTag == nsSVGAtoms::circle)
	rv = NS_NewSVGCircleFrame(aPresShell, aFrame);
  else if (mInner.mTag == nsSVGAtoms::ellipse)
	rv = NS_NewSVGEllipseFrame(aPresShell, aFrame);
  else if (mInner.mTag == nsSVGAtoms::line)
	rv = NS_NewSVGLineFrame(aPresShell, aFrame);
  else if (mInner.mTag == nsSVGAtoms::polygon)
	rv = NS_NewSVGPolygonFrame(aPresShell, aFrame);
  else if (mInner.mTag == nsSVGAtoms::polyline)
	rv = NS_NewSVGPolylineFrame(aPresShell, aFrame);
  else if (mInner.mTag == nsSVGAtoms::g)
	rv = NS_NewSVGGFrame(aPresShell, aFrame);

  return rv;	
}

NS_IMETHODIMP
nsSVGElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
{
  nsSVGElement* it = new nsSVGElement(mInner.mTag);
  if (nsnull == it) {
    return NS_ERROR_OUT_OF_MEMORY;
  }
  mInner.CopyInnerTo((nsIContent *)(nsIXMLContent *)this, &it->mInner, aDeep);
  return it->QueryInterface(kIDOMNodeIID, (void**) aReturn);
}

// nsIStyledContent Implementation
NS_IMETHODIMP
nsSVGElement::GetID(nsIAtom*& aResult) const
{
  printf("nsSVGElement::GetID()\n");
  nsString value;
  GetAttribute(kNameSpaceID_None, nsSVGAtoms::id, value);

  aResult = NS_NewAtom(value); // The NewAtom call does the AddRef.
  return NS_OK;
}
    
NS_IMETHODIMP
nsSVGElement::GetClasses(nsVoidArray& aArray) const
{
  // XXX: TBI
  return NS_OK;
}

NS_IMETHODIMP 
nsSVGElement::HasClass(nsIAtom* aClass) const
{
  // XXX: TBI
  return NS_OK;
}

NS_IMETHODIMP
nsSVGElement::GetContentStyleRules(nsISupportsArray* aRules)
{
  printf("nsSVGElement::GetContentStyleRules()\n");
  return NS_OK;
}
    
NS_IMETHODIMP
nsSVGElement::GetInlineStyleRules(nsISupportsArray* aRules)
{
  printf("nsSVGElement::GetInlineStyleRules()\n");
  nsresult result = NS_ERROR_NULL_POINTER;
  nsIStyleRule* rule = nsnull;
  if (aRules) {
    nsICSSParser * css = nsnull;
    result = nsComponentManager::CreateInstance(kCSSParserCID,
                                                nsnull,
                                                kICSSParserIID,
                                                (void**)&css);
    if (NS_OK != result) {
      return result;
    }

    nsIURI* docURL = nsnull;
    if (nsnull != mInner.mDocument) {
       mInner.mDocument->GetBaseURL(docURL);
    }

    nsAutoString val;
    GetAttribute(kNameSpaceID_None, nsSVGAtoms::style, val);

    result = css->ParseDeclarations(val, docURL, rule);
    NS_IF_RELEASE(docURL);
    NS_RELEASE(css);
    
    if ((NS_OK == result) && (nsnull != rule)) {
      aRules->AppendElement((nsISupports*)rule);
      NS_RELEASE(rule);
    }  
  }
  return result;
}

NS_IMETHODIMP
nsSVGElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 &aHint) const
{
  printf("nsSVGElement::GetMappedAttributeImpact()\n");
  aHint = NS_STYLE_HINT_CONTENT;
  return NS_OK;
}

NS_IMETHODIMP
nsSVGElement::SetBinding(nsIXBLBinding* aBinding) 
{
  mBinding = aBinding; // nsCOMPtr handles addrefing.
  return NS_OK;
}

NS_IMETHODIMP
nsSVGElement::GetBinding(nsIXBLBinding** aResult)
{
  *aResult = mBinding.get();
  NS_IF_ADDREF(*aResult);
  return NS_OK;
}

NS_IMETHODIMP
nsSVGElement::GetBaseTag(nsIAtom** aResult)
{
  if (mBinding)
    return mBinding->GetBaseTag(aResult);
  else
    return NS_OK;
}

